module Locmare
  class Booster
    attr :boost, :manifest
    
    def initialize manifest, item
      @manifest = manifest
      @item = item
      # no check
      # return false unless self.resource.enable?
      if self.extendable?
        @item.extend self.extend_module
        @item.__send__ @manifest.setter_method_name, self.parsed_settings if self.has_extend_model?
      end
    end
    
    # get template from system resource manifest
    def template_path_name
      self.template_manifest.template_items[self.template_module_name]
    end
    
    # get template manifest from system resource manifest
    def template_manifest
      Manifest.manifest.system_resources.templates[@manifest.template_name]
    end
    
    def item_name
      self.template_path_name + '_' + @manifest.item_name_for_extend_model
    end
    
    def enable?
      self.template_manifest.template_items.include? self.template_module_name
    end
    
    # get template module name
    # ex) attributes[hoge_module_name] -> 'Hoge'
    def template_module_name
      @item.attributes[@manifest.column_name_for_template_module]
    end
    
    def settings
      @item.attributes[@manifest.settings_column_name]
    end
    
    # name space for template
    # ex) 'Hoge' -> Hoge
    def template_module
      Object.const_get self.template_module_name
    end
    
    def extendable?
      self.template_module.constants.include? @manifest.extend_module_name.to_sym
    end
    
    # extend module in engine
    # ex) 'SpeechBalloonModule' -> CircleSpeechBalloon::SpeechBalloonModule
    def extend_module
      self.template_module.const_get @manifest.extend_module_name
    end
    
    def has_extend_model?
      !@manifest.extend_model_class_name.blank?
    end
    
    # extend settings data model in engine
    # ex) CircleSpeechBalloon::SpeechBalloon
    def extend_model
      if self.has_extend_model?
        self.template_module.const_get @manifest.extend_model_class_name
      else
        nil
      end
    end
    
    def check_template
      @item.attributes[@manifest.foreign_key].blank? == false
    end
    
    # get template model
    def template_model
      Manifest.manifest.models[@manifest.template_name].classify
    end
    
    # get extend instance
    def extend_item
      if self.extend_model
        @item.__send__ @manifest.getter_method_name
      else
        nil
      end
    end
    
    def parsed_settings
      if self.settings.blank?
        {}
      else
        JSON.parse(self.settings)
      end
    end
    
    def update_settings new_settings
      @item.__send__ @manifest.setter_method_name, new_settings
    end
    
    def supply_default
      self.extend_item.supply_default @item
    end
    
    def overwrite 
      self.extend_item.overwrite @item
    end
    
  end
end

