module Peta
  class Leaf < Content
    self.abstract_class = true
    
    # Dynamic ClassMethods
    
    def self.load_manifest
      super
      return nil if self._skip_load?
      # Class Methods
      pm = Manifest.manifest.models[self.my_peta.parent_item_name].classify
      define_singleton_method("parent_model") do 
        pm
      end
      destm = Manifest.manifest.models[self.my_peta.destination_item_name].classify
      define_singleton_method("destination_model") do 
        destm
      end
      pfk = self.my_peta.parent_item_name + '_id'
      define_singleton_method("binder_key") do 
        pfk
      end
      dest_key = if self.my_peta.destination_item_name
        self.my_peta.destination_item_name + '_id'
      else
        nil
      end
      define_singleton_method("destination_key") do 
        dest_key
      end
      # Instance Methods
    end
    
    def self.element?
      true
    end
    
    def self.root_model
      self.parent_model
    end
    
    def self.binder_model
      self.parent_model
    end
    
    def self.destination_model
      self.parent_model
    end
    
    def root
      self.__send__ self.class.root_model.item_name
    end
    
    def own? operators
      self.root.own? operators
    end
    
    # super return if my item
    def visible? operators
      return false if super == false
      self.root.visible? operators
    end
    
    def self.play_list_order
      self.table_name + '.t'
    end
    
    def self.new_t binder_id
      r = self.max_t(binder_id)
      r.blank? ? 0 : r.to_i + 1
    end
    
    def self.max_t binder_id
      self.where([self.binder_key + ' = ?', binder_id]).maximum(:t)
    end
    
    def self.find_t binder_id, t
      self.where([self.binder_key + ' = ? and t = ?', binder_id, t]).first
    end
    
    def self.collect_t binder_id
      r = self.where([self.binder_key + ' = ?', binder_id]).order('t')
      r.map {|sp| sp.t}
    end
    
    def self.serial? ary
      i = 0
      ary.compact.sort.each do |t|
        break false unless t == i
        i += 1
      end
      ary.compact.size == i
    end
    
    def self.validate_t binder_id
      self.serial?(self.collect_t(binder_id))
    end
    
    def binder_key
      self.class.binder_key
    end
    
    def destination_key
      self.class.destination_key
    end
    
    def binder_id
      self.attributes[self.binder_key]
    end
    
    def destination_id
      if self.destination_key
        self.attributes[self.destination_key]
      else
        nil
      end
    end
    
    def insert_shift
      self.class.where([self.binder_key + ' = ? and t >= ?', self.binder_id, self.t]).update_all('t = t + 1')
    end
    
    def lesser_shift old_t
      self.t = 0 if self.t < 0
      self.class.where([self.binder_key + ' = ? and (t >= ? and t < ?)', self.binder_id, self.t, old_t]).update_all('t = t + 1')
    end
    
    def higher_shift old_t
      nf = self.class.find_t(self.binder_id, self.t)
      max_t = self.class.max_t(self.binder_id).to_i
      self.t = max_t if self.t > max_t
      self.class.where([self.binder_key + ' = ? and (t > ? and t <= ?)', self.binder_id, old_t, self.t]).update_all('t = t - 1')
    end
    
    def update_shift old_t
      if self.t > old_t
        higher_shift old_t
      else
        lesser_shift old_t
      end
    end
    
    def rotate old_t = nil
      if self.new_record?
        if self.t.blank?
          self.t = self.class.new_t self.binder_id
        else
          self.insert_shift
        end
      else
        if self.t.blank?
        else
          self.update_shift old_t
        end
      end
    end
    
    def store operators, old_t = nil
      res = false
      self.class.transaction do
        case self.allow? operators
        when true
          self.rotate old_t
        when false
          raise ActiveRecord::Forbidden
        else
        end
        res = self.save
        raise ActiveRecord::Rollback unless res
        res = self.class.validate_t(self.binder_id) 
        unless res
          self.errors.add :t, 'unserialized'
          raise ActiveRecord::Rollback 
        end
      end
      res
    end
    
    # destroy and shorten
    def destroy
      res = false
      self.class.transaction do
        # renumber t
        self.class.update_all('t = t - 1', 
          [self.binder_key + ' = ? and (t > ?)', self.binder_id, self.t]
        )
        res = super
        raise ActiveRecord::Rollback unless res
      end
      res
    end
    
  end
end

