# -*- encoding: utf-8 -*-
#ライセンスグループ
require 'spec_helper'

describe LicenseGroup do
  include ManifestsMacros
  before do
    @model = LicenseGroup
  end
  
  describe 'ペタナイズに於いて' do
    it '素材のライセンスを制御するためのテンプレートである' do
      expect(@model.ancestors.include?(Peta::Template)).to eq true
    end
    it '親モデルは持たない' do
      expect(@model.my_peta.parent_model_name).to be_blank
    end
    describe 'ブーストに於いて' do
      it '特筆すべき注意事項はない' do
        expect(@model.my_peta.boost.keys).to be_empty
      end
    end
    
  end
  
  describe 'コントローラに於いて' do
    before do
      @controller = @model.my_controller
    end
    it 'ライセンスグループのためのコントローラである' do
      expect(@model.item_name).to eq 'license_group'
    end
    describe 'actionに於いて' do
      before do
        @actions = @controller.actions
        @actions_names = ['index', 'show', 
          'count', 
          'new', 'edit', 'create', 'update', 'destroy']
      end
      it 'これらのアクションが定義されている' do
        set_actions? @actions, @actions_names
      end
      describe 'baseに於いて' do
      end
      describe 'indexに於いて' do
        before do
          @action = @actions['index']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # ライセンスグループの公開リストを返すように定義している
          set_list_action? @action, 'license_group', 'public'
        end
      end
      describe 'countに於いて' do
        before do
          @action = @actions['count']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          #ライセンスグループの公開リストのカウントを返すように定義している
          set_list_action? @action, 'license_group', 'public'
        end
      end
      describe 'showに於いて' do
        before do
          @action = @actions['show']
        end
        it 'showタイプのアクションである' do
          expect(@action.type).to eq 'show'
        end
      end
      describe 'newに於いて' do
        before do
          @action = @actions['new']
        end
        it 'newタイプのアクションである' do
          expect(@action.type).to eq 'new'
        end
        it 'type' do
          expect(@action.item_name).to eq 'license_group'
        end
      end
      describe 'editに於いて' do
        before do
          @action = @actions['edit']
        end
        it 'editタイプのアクションである' do
          expect(@action.type).to eq 'edit'
        end
        it 'type' do
          expect(@action.item_name).to eq 'license_group'
        end
      end
      describe 'createに於いて' do
        before do
          @action = @actions['create']
        end
        it 'createタイプのアクションである' do
          expect(@action.type).to eq 'create'
        end
        it 'type' do
          expect(@action.item_name).to eq 'license_group'
        end
      end
      describe 'updateに於いて' do
        before do
          @action = @actions['update']
        end
        it 'updateタイプのアクションである' do
          expect(@action.type).to eq 'update'
        end
        it 'type' do
          expect(@action.item_name).to eq 'license_group'
        end
      end
      describe 'destroyに於いて' do
        before do
          @action = @actions['destroy']
        end
        it 'destroyタイプのアクションである' do
          expect(@action.type).to eq 'destroy'
        end
        it 'type' do
          expect(@action.item_name).to eq 'license_group'
        end
      end
    end
    
  end
  
  describe 'モデルに於いて' do
    before do
      @my_manifest = @model.my_manifest
    end
    it 'サポート機能が動作している' do
      expect(@my_manifest.model_name).to eq 'license_group'
      expect(@my_manifest.classify).to eq @model
      expect(@my_manifest.table_name).to eq 'license_groups'
    end
    
    describe 'associationsに於いて' do
      describe 'belongs_toに於いて' do
        before do
          @belongs_to = @my_manifest.associations.belongs_to
          @names = []
        end
        it 'これらのモデルに所属している' do
          set_associations? @belongs_to, @names
        end
      end
      describe 'has_manyに於いて' do
        before do
          @has_many = @model.my_manifest.associations.has_many
          @names = ['resource_pictures', 'licenses']
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_many, @names
        end
        describe 'resource_picturesに於いて' do
          before do
            @mani = @has_many['resource_pictures']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'resource_picture'
            expect(@mani.foreign_key).to eq 'license_group_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq ResourcePicture
          end
        end
        describe 'licensesに於いて' do
          before do
            @mani = @has_many['licenses']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'license'
            expect(@mani.foreign_key).to eq 'license_group_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq License
          end
        end
      end
    end
    
    describe 'attributesに於いて' do
      before do
        @attributes = @model.my_manifest.attributes
      end
      it 'これらのカラムを定義している' do
        set_attributes? @attributes, 
          ['id', 'name', 'module_name', 'caption', 'url', 
          'created_at', 'updated_at']
      end
      describe 'nameに於いて' do
        before do
          @attribute = @attributes['name']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'module_nameに於いて' do
        before do
          @attribute = @attributes['module_name']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'captionに於いて' do
        before do
          @attribute = @attributes['caption']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'urlに於いて' do
        before do
          @attribute = @attributes['url']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
    end
    
  end
  
end
