# -*- encoding: utf-8 -*-
require 'spec_helper'
#用紙

describe Sheet do
  include ManifestsMacros
  before do
    @model = Sheet
  end
  
  describe 'ペタナイズに於いて' do
    it 'ルートである' do
      expect(@model.ancestors.include?(Peta::Root)).to eq true
    end
    it '親モデルは持たない' do
      expect(@model.my_peta.parent_model_name).to be_blank
    end
    describe 'ブーストに於いて' do
      it '特筆すべき注意事項はない' do
        expect(@model.my_peta.boost.keys).to be_empty
      end
    end
    
  end
  
  describe 'コントローラに於いて' do
    before do
      @controller = @model.my_controller
    end
    it '用紙のためのコントローラである' do
      expect(@model.item_name).to eq 'sheet'
    end
    describe 'actionに於いて' do
      before do
        @actions = @controller.actions
        @actions_names = ['index', 'by_story', 'by_author', 'by_panel', 'play', 'show', 
          'count', 'count_by_story', 'count_by_author', 'count_by_panel', 
          'new', 'edit', 'create', 'update', 'destroy']
      end
      it 'これらのアクションが定義されている' do
        set_actions? @actions, @actions_names
      end
      describe 'baseに於いて' do
      end
      describe 'indexに於いて' do
        before do
          @action = @actions['index']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # 用紙の公開リストを返すように定義している
          set_list_action? @action, 'sheet', 'public'
        end
      end
      describe 'by_storyに於いて' do
        before do
          @action = @actions['by_story']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # 用紙のストーリーフィルタリストを返すように定義している
          set_list_action? @action, 'sheet', 'by_story'
        end
      end
      describe 'by_authorに於いて' do
        before do
          @action = @actions['by_author']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # 用紙の作家フィルタリストを返すように定義している
          set_list_action? @action, 'sheet', 'by_author'
        end
      end
      describe 'by_panelに於いて' do
        before do
          @action = @actions['by_panel']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          #用紙のコマフィルタリストを返すように定義している
          set_list_action? @action, 'sheet', 'by_panel'
        end
      end
      describe 'showに於いて' do
        before do
          @action = @actions['show']
        end
        it 'showタイプのアクションである' do
          expect(@action.type).to eq 'show'
        end
      end
      describe 'countに於いて' do
        before do
          @action = @actions['count']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          #用紙の公開リストのカウントを返すように定義している
          set_list_action? @action, 'sheet', 'public'
        end
      end
      describe 'count_by_storyに於いて' do
        before do
          @action = @actions['count_by_story']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # 用紙のストーリーフィルタリストのカウントを返すように定義している
          set_list_action? @action, 'sheet', 'by_story'
        end
      end
      describe 'count_by_authorに於いて' do
        before do
          @action = @actions['count_by_author']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # 用紙の作家フィルタリストのカウントを返すように定義している
          set_list_action? @action, 'sheet', 'by_author'
        end
      end
      describe 'count_by_panelに於いて' do
        before do
          @action = @actions['count_by_panel']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # 用紙のコマフィルタリストのカウントを返すように定義している
          set_list_action? @action, 'sheet', 'by_panel'
        end
      end
      describe 'newに於いて' do
        before do
          @action = @actions['new']
        end
        it 'newタイプのアクションである' do
          expect(@action.type).to eq 'new'
        end
        it 'type' do
          expect(@action.item_name).to eq 'sheet'
        end
      end
      describe 'editに於いて' do
        before do
          @action = @actions['edit']
        end
        it 'editタイプのアクションである' do
          expect(@action.type).to eq 'edit'
        end
        it 'type' do
          expect(@action.item_name).to eq 'sheet'
        end
      end
      describe 'createに於いて' do
        before do
          @action = @actions['create']
        end
        it 'createタイプのアクションである' do
          expect(@action.type).to eq 'create'
        end
        it 'type' do
          expect(@action.item_name).to eq 'sheet'
        end
      end
      describe 'updateに於いて' do
        before do
          @action = @actions['update']
        end
        it 'updateタイプのアクションである' do
          expect(@action.type).to eq 'update'
        end
        it 'type' do
          expect(@action.item_name).to eq 'sheet'
        end
      end
      describe 'destroyに於いて' do
        before do
          @action = @actions['destroy']
        end
        it 'destroyタイプのアクションである' do
          expect(@action.type).to eq 'destroy'
        end
        it 'type' do
          expect(@action.item_name).to eq 'sheet'
        end
      end
    end
    
  end
  
  describe 'モデルに於いて' do
    before do
      @my_manifest = @model.my_manifest
    end
    it 'サポート機能が動作している' do
      expect(@my_manifest.model_name).to eq 'sheet'
      expect(@my_manifest.classify).to eq @model
      expect(@my_manifest.table_name).to eq 'sheets'
    end
    
    describe 'associationsに於いて' do
      describe 'belongs_toに於いて' do
        before do
          @belongs_to = @my_manifest.associations.belongs_to
          @names = ['author']
        end
        it 'これらのモデルに所属している' do
          set_associations? @belongs_to, @names
        end
        describe 'authorに於いて' do
          before do
            @mani = @belongs_to['author']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'author'
            expect(@mani.id_column).to eq 'author_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq Author
          end
        end
      end
      describe 'has_manyに於いて' do
        before do
          @has_many = @model.my_manifest.associations.has_many
          @names = ['story_sheets', 'panels', 'sheet_panels', 'panels']
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_many, @names
        end
        describe 'story_sheetsに於いて' do
          before do
            @mani = @has_many['story_sheets']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'story_sheet'
            expect(@mani.foreign_key).to eq 'sheet_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq StorySheet
          end
        end
        describe 'storiesに於いて' do
          before do
            @mani = @has_many['stories']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'story'
            expect(@mani.foreign_key).to eq 'sheet_id'
            expect(@mani.through).to eq 'story_sheets'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be true
            expect(@mani.model).to eq Story
          end
        end
        describe 'sheet_panelsに於いて' do
          before do
            @mani = @has_many['sheet_panels']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'sheet_panel'
            expect(@mani.foreign_key).to eq 'sheet_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq SheetPanel
          end
        end
        describe 'panelsに於いて' do
          before do
            @mani = @has_many['panels']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'panel'
            expect(@mani.foreign_key).to eq 'sheet_id'
            expect(@mani.through).to eq 'sheet_panels'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be true
            expect(@mani.model).to eq Panel
          end
        end
      end
    end
    
    describe 'attributesに於いて' do
      before do
        @attributes = @model.my_manifest.attributes
      end
      it 'これらのカラムを定義している' do
        set_attributes? @attributes, 
          ['id', 'caption', 'width', 'height', 'visible', 'author_id', 
          'created_at', 'updated_at']
      end
      describe 'captionに於いて' do
        before do
          @attribute = @attributes['caption']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'widthに於いて' do
        before do
          @attribute = @attributes['width']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'heightに於いて' do
        before do
          @attribute = @attributes['height']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'visibleに於いて' do
        before do
          @attribute = @attributes['visible']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
        it '選択肢を定数から取得できるように定義している' do
          expect(@attribute.source.type).to eq 'magic_number'
          expect(@attribute.source.select_item_name).to eq 'sheet_visible_items'
        end
      end
      describe 'author_idに於いて' do
        before do
          @attribute = @attributes['author_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
    end
    
  end
  
end
