# -*- encoding: utf-8 -*-
require 'spec_helper'
#フキダシテンプレート
describe SpeechBalloonTemplate do
  include ManifestsMacros
  before do
    @model = SpeechBalloonTemplate
  end
  
  describe 'ペタナイズに於いて' do
    it 'スクコマを閉じるためのバインダーである' do
      expect(@model.ancestors.include?(Peta::Template)).to eq true
    end
    it '親モデルは持たない' do
      expect(@model.my_peta.parent_model_name).to be_blank
    end
    describe 'ブーストに於いて' do
      it '特筆すべき注意事項はない' do
        expect(@model.my_peta.boost.keys).to be_empty
      end
    end
    
  end
  
  describe 'コントローラに於いて' do
    before do
      @controller = @model.my_controller
    end
    it 'フキダシテンプレートのためのコントローラである' do
      expect(@model.item_name).to eq 'speech_balloon_template'
    end
    describe 'actionに於いて' do
      before do
        @actions = @controller.actions
        @actions_names = ['index', 'by_panel', 'show', 
          'count', 'count_by_panel', 
          'new', 'edit', 'create', 'update', 'destroy']
      end
      it 'これらのアクションが定義されている' do
        set_actions? @actions, @actions_names
      end
      describe 'baseに於いて' do
      end
      describe 'indexに於いて' do
        before do
          @action = @actions['index']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # フキダシテンプレートの公開リストを返すように定義している
          set_list_action? @action, 'speech_balloon_template', 'public'
        end
      end
      describe 'by_panelに於いて' do
        before do
          @action = @actions['by_panel']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          #フキダシテンプレートのコマフィルタリストを返すように定義している
          set_list_action? @action, 'speech_balloon_template', 'by_panel'
        end
      end
      describe 'countに於いて' do
        before do
          @action = @actions['count']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          #フキダシテンプレートの公開リストのカウントを返すように定義している
          set_list_action? @action, 'speech_balloon_template', 'public'
        end
      end
      describe 'showに於いて' do
        before do
          @action = @actions['show']
        end
        it 'showタイプのアクションである' do
          expect(@action.type).to eq 'show'
        end
      end
      describe 'count_by_panelに於いて' do
        before do
          @action = @actions['count_by_panel']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # フキダシテンプレートのコマフィルタリストのカウントを返すように定義している
          set_list_action? @action, 'speech_balloon_template', 'by_panel'
        end
      end
      describe 'newに於いて' do
        before do
          @action = @actions['new']
        end
        it 'newタイプのアクションである' do
          expect(@action.type).to eq 'new'
        end
        it 'type' do
          expect(@action.item_name).to eq 'speech_balloon_template'
        end
      end
      describe 'editに於いて' do
        before do
          @action = @actions['edit']
        end
        it 'editタイプのアクションである' do
          expect(@action.type).to eq 'edit'
        end
        it 'type' do
          expect(@action.item_name).to eq 'speech_balloon_template'
        end
      end
      describe 'createに於いて' do
        before do
          @action = @actions['create']
        end
        it 'createタイプのアクションである' do
          expect(@action.type).to eq 'create'
        end
        it 'type' do
          expect(@action.item_name).to eq 'speech_balloon_template'
        end
      end
      describe 'updateに於いて' do
        before do
          @action = @actions['update']
        end
        it 'updateタイプのアクションである' do
          expect(@action.type).to eq 'update'
        end
        it 'type' do
          expect(@action.item_name).to eq 'speech_balloon_template'
        end
      end
      describe 'destroyに於いて' do
        before do
          @action = @actions['destroy']
        end
        it 'destroyタイプのアクションである' do
          expect(@action.type).to eq 'destroy'
        end
        it 'type' do
          expect(@action.item_name).to eq 'speech_balloon_template'
        end
      end
    end
    
  end
  
  describe 'モデルに於いて' do
    before do
      @my_manifest = @model.my_manifest
    end
    it 'サポート機能が動作している' do
      expect(@my_manifest.model_name).to eq 'speech_balloon_template'
      expect(@my_manifest.classify).to eq @model
      expect(@my_manifest.table_name).to eq 'speech_balloon_templates'
    end
    
    describe 'associationsに於いて' do
      describe 'belongs_toに於いて' do
        before do
          @belongs_to = @my_manifest.associations.belongs_to
          @names = ['system_picture']
        end
        it 'これらのモデルに所属している' do
          set_associations? @belongs_to, @names
        end
        describe 'system_pictureに於いて' do
          before do
            @mani = @belongs_to['system_picture']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'system_picture'
            expect(@mani.id_column).to eq 'system_picture_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq SystemPicture
          end
        end
      end
      describe 'has_manyに於いて' do
        before do
          @has_many = @model.my_manifest.associations.has_many
          @names = ['speech_balloons', 'panels']
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_many, @names
        end
        describe 'speech_balloonsに於いて' do
          before do
            @mani = @has_many['speech_balloons']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'speech_balloon'
            expect(@mani.foreign_key).to eq 'speech_balloon_template_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq SpeechBalloon
          end
        end
        describe 'panelsに於いて' do
          before do
            @mani = @has_many['panels']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'panel'
            expect(@mani.foreign_key).to eq 'speech_balloon_template_id'
            expect(@mani.through).to eq 'speech_balloons'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be true
            expect(@mani.model).to eq Panel
          end
        end
      end
    end
    
    describe 'attributesに於いて' do
      before do
        @attributes = @model.my_manifest.attributes
      end
      it 'これらのカラムを定義している' do
        set_attributes? @attributes, 
          ['id', 'name', 'module_name', 'caption', 't', 'system_picture_id', 'settings', 
          'created_at', 'updated_at']
      end
      describe 'nameに於いて' do
        before do
          @attribute = @attributes['name']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'module_nameに於いて' do
        before do
          @attribute = @attributes['module_name']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'captionに於いて' do
        before do
          @attribute = @attributes['caption']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'tに於いて' do
        before do
          @attribute = @attributes['t']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'system_picture_idに於いて' do
        before do
          @attribute = @attributes['system_picture_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'settingsに於いて' do
        before do
          @attribute = @attributes['settings']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
    end
    
  end
  
end
