# -*- encoding: utf-8 -*-
require 'spec_helper'
#スクロール

describe Scroll do
  include ItemMacros
  before do
    @admin =FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @author_yas =  @user_yas.author
    
    @model = Scroll
    @item =  FactoryGirl.create :scroll_hello_with_scroll_panels, author: @author_yas
  end
  
  describe '生成メソッドに於いて' do
    before do
    end
    
    describe '文字コード検証に於いて' do
      it 'Shift JISなら失敗する' do
        @text_column_names = ['title', 'description']
        valid_encode? @item, @text_column_names
      end
    end
    
    describe 'child_modelsに於いて' do
      it 'すべてのリーフを取得できる' do
        define_child_models? @model, [ScrollPanel]
      end
    end
    
    describe 'child_element_namesに於いて' do
      it 'すべてのリーフを取得できる' do
        define_child_element_names? @model, ['scroll_panels']
      end
    end
    
  end
  
  describe 'myに於いて' do
    it '自分のマニフェストを取得するためのショートカットが設定できている' do
      define_my_manifest? @model
    end
    
    it 'アイテムの種別を判定するためのフラグが設定できている' do
      @subjects = {
        :item? => true,
        :content? => true,
        :element? => false,
        :root? => false,
      }
      set_name? @model, @subjects
    end
    
    it '名前の活用に於いて' do
      @subjects = {
        :singular => 'Scroll',
        :plural => 'Scrolls',
        :item_name => 'scroll',
        #:model_name => 'Scroll',
        :path_name => 'scrolls',
        :table_name => 'scrolls',
        :pickup_item_name => 'scroll',
        :pickup_column_name => 'id',
      }
      # クラスメソッドで名前の活用ができているか
      set_name? @model, @subjects
      # インスタンスメソッドからクラスメソッドを呼び出すショートカットが利用できるか
      set_name? @item, @subjects
    end
  end
    
  describe 'listに於いて' do
    it 'list' do
      expect(@model.parent_model).to be nil
      expect(@model.list_where).to be_blank
      expect(@model.list_order).to eq 'scrolls.updated_at desc'
    end
  end
  
  describe 'extendに於いて' do
    before do
    end
    
    it 'それぞれのカラムは拡張カラムなのか' do
      @subjects = {
        :title => false,
        :description => false,
        :visible => false,
        :author_id => false,
      }
      set_extend_column? @model, @subjects
      set_extend_column? @item, @subjects
    end
    
    it 'カラム名からブーストマニフェストを取得できている' do
      @subjects = {
      }
      set_extend_column? @model, @subjects
    end
    
    it 'Postレベルでブーストしてブースターを確認する' do
      @names = []
      set_boosters? @item, @names
    end
    
    it '拡張データを畳み込める' do
      @extend_column_names = []
      @attr = {}
      fold_extend_settings? @model, @attr, @extend_column_names
    end
    
  end
  
  describe 'めんどいからチェックしない機能に於いて' do
    it 'とりあえず呼び出すところまでは確認しておく' do
      define_test @item
    end
    # 
    # list_opt, list_json_opt, show_opt, show_json_opt, form_template, form_name
    # supply_default, overwrite, visible?
  end
  
end

