module ItemMacros
  # 文字コード検証
  def valid_encode? item, text_column_names
    text_column_names.each do |name|
      expect(item.class.valid_encode_columns).to include name
      item.attributes[name] = "\x83G\x83r\x83]\x83D"
      # バージョンが上がって問題なくなったかもしれない
      #expect { item.valid_encode }.to raise_error(Pettanr::BadRequest)
    end
  end
  
  def define_child_models? model, models
    expect(model.child_models.size).to eq models.size
    models.each do |m|
      expect(model.child_models).to include m
    end
  end
  
  def define_child_element_names? model, item_names
    expect(model.child_models.size).to eq item_names.size
    item_names.each do |m|
      expect(model.child_element_names).to include m
    end
  end
  
  def define_my_manifest? model
    expect(model.my_peta).to eq Manifest.manifest.items[model.item_name]
    expect(model.my_controller).to eq Manifest.manifest.controllers[model.table_name]
    expect(model.my_manifest).to eq Manifest.manifest.models[model.item_name]
  end
  
  # マニフェストを取得するためのショートカット
  def define_my_manifest? model
    expect(model.my_peta).to eq Manifest.manifest.items[model.item_name]
    expect(model.my_controller).to eq Manifest.manifest.controllers[model.table_name]
    expect(model.my_manifest).to eq Manifest.manifest.models[model.item_name]
  end
  
  # 名前の活用が正しく変化していることを確認している
  def set_name? item, subjects
    subjects.each do |name, value|
      expect(item.__send__(name)).to eq value
    end
  end
  
  # 拡張カラムなのかを確認している
  def set_extend_column? model, subjects
    subjects.each do |name, value|
      expect(model.extend_column?(name)).to eq value
    end
  end
  
  # ブースターが用意されているか、名前だけでも確認する
  def set_boosters? item, names
    item.boosts 'post'
    item.boosters.keys.each do |name|
      expect(name).to include names
    end
  end
  
  # 拡張データを畳み込めるか確認する
  def fold_extend_settings? model, attr, extend_column_names
    model.fold_extend_settings attr
    extend_column_names.each do |name|
      expect(attr[name]).to be_kind_of String
    end
  end
  
  def define_test item
    expect {expect(item.dom_id)}.not_to raise_error
    expect {expect(item.editize?)}.not_to raise_error
    expect {expect(item.dom_pool_type)}.not_to raise_error
    expect {expect(item.merge_dom_item_id({}))}.not_to raise_error
    expect {expect(item.tag_attributes)}.not_to raise_error
    expect {expect(item.field_tag_attributes(:hoge))}.not_to raise_error
    expect {expect(item.post_attribute_key)}.not_to raise_error
  end
  
end

module ContentMacros
  # ぶら下がるそれぞれのリーフを取得
  def define_leaf_items? item, leaf_models
    expect(item.respond_to?(:leaf_items)).to be true
    leaf_models.each do |model|
      expect(item.leaf_items(model)).to_not be_empty
      expect(item.leaf_items(model).first.class).to eq model
    end
  end
  
  def define_leafs_items? item, leaf_models
    expect(item.respond_to?(:leafs_items)).to be true
    expect(item.leafs_items).to_not be_empty
    item.leafs_items.map {|i| i.class }.each do |model|
      expect(leaf_models).to include model
    end
  end
  
end

module BinderMacros
  # ぶら下がるそれぞれのリーフを取得
  def define_leaf_items? item, leaf_models
    expect(item.respond_to?(:leaf_items)).to be true
    leaf_models.each do |model|
      expect(item.leaf_items(model)).to_not be_empty
      expect(item.leaf_items(model).first.class).to eq model
    end
  end
  
  def define_leafs_items? item, leaf_models
    expect(item.respond_to?(:leafs_items)).to be true
    expect(item.leafs_items).to_not be_empty
    item.leafs_items.map {|i| i.class }.each do |model|
      expect(leaf_models).to include model
    end
  end
  
end

module LeafMacros
  def define_test item
    expect {expect(item.new_t(0))}.not_to raise_error
  end
  
end

module RootMacros
  # ぶら下がるそれぞれのリーフを取得
  def define_leaf_items? item, leaf_models
    expect(item.respond_to?(:leaf_items)).to be true
    leaf_models.each do |model|
      expect(item.leaf_items(model)).to_not be_empty
      expect(item.leaf_items(model).first.class).to eq model
    end
  end
  
  def define_leafs_items? item, leaf_models
    expect(item.respond_to?(:leafs_items)).to be true
    expect(item.leafs_items).to_not be_empty
    item.leafs_items.map {|i| i.class }.each do |model|
      expect(leaf_models).to include model
    end
  end
  
end

module ElementMacros
  # ぶら下がるそれぞれのリーフを取得
  def define_leaf_items? item, leaf_models
    expect(item.respond_to?(:leaf_items)).to be true
    leaf_models.each do |model|
      expect(item.leaf_items(model)).to_not be_empty
      expect(item.leaf_items(model).first.class).to eq model
    end
  end
  
  def define_leafs_items? item, leaf_models
    expect(item.respond_to?(:leafs_items)).to be true
    expect(item.leafs_items).to_not be_empty
    item.leafs_items.map {|i| i.class }.each do |model|
      expect(leaf_models).to include model
    end
  end
  
end

module OwnerMacros
  # ぶら下がるそれぞれのリーフを取得
  def define_leaf_items? item, leaf_models
    expect(item.respond_to?(:leaf_items)).to be true
    leaf_models.each do |model|
      expect(item.leaf_items(model)).to_not be_empty
      expect(item.leaf_items(model).first.class).to eq model
    end
  end
  
  def define_leafs_items? item, leaf_models
    expect(item.respond_to?(:leafs_items)).to be true
    expect(item.leafs_items).to_not be_empty
    item.leafs_items.map {|i| i.class }.each do |model|
      expect(leaf_models).to include model
    end
  end
  
end

module SystemResourceMacros
  # ぶら下がるそれぞれのリーフを取得
  def define_leaf_items? item, leaf_models
    expect(item.respond_to?(:leaf_items)).to be true
    leaf_models.each do |model|
      expect(item.leaf_items(model)).to_not be_empty
      expect(item.leaf_items(model).first.class).to eq model
    end
  end
  
  def define_leafs_items? item, leaf_models
    expect(item.respond_to?(:leafs_items)).to be true
    expect(item.leafs_items).to_not be_empty
    item.leafs_items.map {|i| i.class }.each do |model|
      expect(leaf_models).to include model
    end
  end
  
end

module TemplateMacros
  # ぶら下がるそれぞれのリーフを取得
  def define_leaf_items? item, leaf_models
    expect(item.respond_to?(:leaf_items)).to be true
    leaf_models.each do |model|
      expect(item.leaf_items(model)).to_not be_empty
      expect(item.leaf_items(model).first.class).to eq model
    end
  end
  
  def define_leafs_items? item, leaf_models
    expect(item.respond_to?(:leafs_items)).to be true
    expect(item.leafs_items).to_not be_empty
    item.leafs_items.map {|i| i.class }.each do |model|
      expect(leaf_models).to include model
    end
  end
  
end

